{{      
************************************************
* APA102 LED SPI Spin Demo                v0.1 *
*                                              *
* Based on:                                    *
* Propeller SPI Spin Demo                 v1.0 *
* Author: Beau Schwabe                         *
* Copyright (c) 2009 Parallax                  *
* See end of file for terms of use.            *
************************************************
}}

CON

'  _clkmode = xtal1 + pll16x                           
'  _xinfreq = 5_000_000
  
' For Cluso P8XBlade2, running @96MHz
    _clkmode = xtal1 + pll8x                           
    _xinfreq = 12_000_000


OBJ

  SPI : "SPI_Spin"               ''The Standalone SPI Spin engine
  Ser : "FullDuplexSerial"       ''Used in this DEMO for Debug

CON

  start_frame   = $00000000     '' min of 32 bits of zero to start full string of RGB LEDs
  end_frame     = $FFFFFFFF     '' DEBUG: 32 bits of ones to end full string of RGB LEDs
                                ''
                                '' The only function of the “End frame” is to supply
                                '' more clock pulses to the string until the data has
                                '' permeated to the last LED.
                                '' The number of clock pulses required is exactly half
                                '' the total number of LEDs in the string. The
                                '' recommended end frame length of 32 is only
                                '' sufficient for strings up to 64 LEDs
                                '' Change this constant, accordingly!!!
                                
  led_count     = 72            '' DEBUG: try a small number of LEDs for testing, increase for release

  red_led_frame = $FF0000FF     '' First 3 bits: MSB must be a"1" followed by two arbitrary bits
                                '' Next 5 bits: Global LED brightness value: 0-31
                                '' Next 8 bits: Blue RGB value
                                '' Next 8 bits: Green RGB value
                                '' Next 8 bits: Red RGB value

  Blue_led_frame   = $FFFF0000  '' First 3 bits: MSB must be a"1" followed by two arbitrary bits
  Green_led_frame  = $FF00FF00  '' First 3 bits: MSB must be a"1" followed by two arbitrary bits
  White_led_frame  = $FFFFFFFF  '' First 3 bits: MSB must be a"1" followed by two arbitrary bits
  Grey_led_frame   = $FF202020  '' First 3 bits: MSB must be a"1" followed by two arbitrary bits
  Aqua_led_frame   = $FF609900  '' First 3 bits: MSB must be a"1" followed by two arbitrary bits
  Dark_red_frame   = $FF000005  '' First 3 bits: MSB must be a"1" followed by two arbitrary bits
  Dark_blue_frame  = $FF050000  '' First 3 bits: MSB must be a"1" followed by two arbitrary bits
  Dark_green_frame = $FF000500  '' First 3 bits: MSB must be a"1" followed by two arbitrary bits

  off_led_frame    = $E0000000  '' shut off LED
                                ''
  end_byte = $00


PUB APA102_DEMO|DO,CO,i
{{
    
--------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------
      
--------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------

APA102C Schematic:

               ┌─────────┐       |        ┌─────────┐
  P0──>  ──DI──┤1       6├──D0  [|] ──DI──┤1       6├──D0
               │         │       |        │         │
  P1──>  ──CI──┤2 LED 1  ├──C0  [|] ──CI──┤2 LED 2 5├──C0   ──> Next LED...
               │         │       |        │         │
  GND──> ──GND─┤3   VCC 4├──┐   [|] ──GND─┤3   VCC 4├─┐
               └─────────┘  │    |        └─────────┘ │
  +5V──> ───────────────────┘─────────────────────────┘──>
          
}}

'' -----[ Initialization ]--------------------------------------------------

''Serial communication Setup
    Ser.start(31, 30, 0, 115200)  '' Initialize serial communication to the PC through the USB connector
                                  '' To view Serial data on the PC use the Parallax Serial Terminal (PST) program.

    Ser.str(string("APA102C Startup"))
    Ser.tx(13)
    Ser.tx(13)

''SPI Setup
  ''SPI.start(ClockDelay, ClockState)
    SPI.start(15,1)           '' Initialize SPI Engine with Clock Delay of 15us and Clock State of 1


''APA102C Setup
      DO  := 3                  '' Set APA102C Data Pin, goes to 1st LED's DI (data in) pin
      CO  := 4                  '' Set APA102C Clock Pin, goes to 1st LED's CI (cloack in) pin
           ''
      Ser.str(string("APA102C Test!!"))
      Ser.tx(13)
      Ser.tx(13)

      waitcnt(cnt+clkfreq*10/100)                      '' Pause for 5s

      repeat
       
       Loop(DO, CO) 
                         
      
PUB Loop(loopDO, loopCO) | i

''    Bluish Green                                                                            
      SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, start_frame)        '' start frame
      Ser.str(string("Bluish Green - LEDs on"))
      Ser.tx(13)
      Ser.tx(13)

      repeat led_count
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, green_led_frame)  '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, Blue_led_frame)   '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, green_led_frame)  '' test Red RGB frame
                                                                ''
      i := 0
      repeat while i < led_count * 5
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,8, end_byte)          '' end frame
        i += 3

      waitcnt(cnt+clkfreq*100/100)                      '' Pause for 1s

''    RedGreenBlue
      SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, start_frame)        '' start frame

      Ser.str(string("Red - LEDs on"))
      Ser.tx(13)
      Ser.tx(13)
      repeat led_count / 3
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, red_led_frame)    '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, Blue_led_frame)  '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, red_led_frame)   '' test Red RGB frame
                                                                    
      i := 0
      repeat while i < led_count
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,8, end_byte)          '' end frame
        i += 3

      waitcnt(cnt+clkfreq*10/100)                      '' Pause for 1s

''    Do something interesting!!!
      SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, start_frame)        '' start frame
    
      Ser.str(string("Red & Blue - LEDs on"))
      Ser.tx(13)
      Ser.tx(13)
      repeat led_count / 3
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, green_led_frame)    '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, blue_led_frame)    '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, off_led_frame)   '' test Red RGB frame

      i := 0
      repeat while i < led_count
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,8, end_byte)          '' end frame
        i += 3

      waitcnt(cnt+clkfreq*10/100)                       '' Pause for 1s

''    Do something interesting!!!
      SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, start_frame)        '' start frame
    
      Ser.str(string("Red & Blue - LEDs on"))
      Ser.tx(13)
      Ser.tx(13)
      repeat led_count / 3
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, red_led_frame)    '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, green_led_frame)    '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, red_led_frame)   '' test Red RGB frame

      i := 0
      repeat while i < led_count
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,8, end_byte)          '' end frame
        i += 3

      waitcnt(cnt+clkfreq*10/100)                       '' Pause for 1s
''        
      Ser.str(string("Low RGB - LEDs on"))
      Ser.tx(13)
      Ser.tx(13)
      
      SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, start_frame)        '' start frame
      repeat led_count / 3
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, Dark_red_frame)   '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, Dark_green_frame) '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, Dark_blue_frame)  '' test Red RGB frame
                                                               ''
      i := 0
      repeat while i < led_count
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,8, end_byte)          '' end frame
        i += 3
      waitcnt(cnt+clkfreq*10/100)                    '' Pause for 1s

''        
      Ser.str(string("All LEDs Off"))
      Ser.tx(13)
      Ser.tx(13)
      
      SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, start_frame)        '' start frame
      repeat led_count
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, off_led_frame)    '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, off_led_frame)    '' test Red RGB frame
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,32, off_led_frame)    '' test Red RGB frame
                                                               ''
      i := 0
      repeat while i < led_count
        SPI.SHIFTOUT(loopDO, loopCO, SPI#MSBFIRST ,8, end_byte)          '' end frame
        i += 3
      waitcnt(cnt+clkfreq*100/100)                    '' Pause for 5s



PUB HIGH(Pin)
    dira[Pin]~~
    outa[Pin]~~
         
PUB LOW(Pin)
    dira[Pin]~~
    outa[Pin]~
{{
┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │                                                            
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │ 
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}    